package meerkat.mixer;

import meerkat.protobuf.Crypto;
import meerkat.protobuf.Mixing;

import java.io.OutputStream;


/**
 * Created by Tzlil on 1/18/2016.
 * implements meerkat.crypto.mixnet.MixerOutput interface
 * container for meerkat.mixer.network.mix result.
 */
public class MixerOutput implements meerkat.crypto.mixnet.MixerOutput {
    private final Mixing.Mix2Proof[][] proofs;
    private final Crypto.RerandomizableEncryptedMessage[][] encryptedMessages;
    private final int logN;

    /**
     * constructor
     * @param logN log (base 2) of the number of votes
     * @param encryptedMessages at level 0 , contains the original encrypted votes
     *                          at each other level contains the re encrypted votes
     * @param proofs in each cell (level,switch) contains the match zero knowledge proof
     */
    public MixerOutput(int logN, Mixing.Mix2Proof[][] proofs,
                       Crypto.RerandomizableEncryptedMessage[][] encryptedMessages) {
        this.proofs = proofs;
        this.encryptedMessages = encryptedMessages;
        this.logN = logN;
    }

    @Override
    public Mixing.Mix2Proof[][] getProofs() {
        return proofs;
    }

    @Override
    public Crypto.RerandomizableEncryptedMessage[][] getEncryptedMessages() {
        return encryptedMessages;
    }

    @Override
    public int getLogN() {
        return logN;
    }

    @Override
    public int getNumLayers() {
        return 2 * logN - 1;
    }
}
