package meerkat.mixer.proofs.generic;

import com.google.protobuf.Message;
import meerkat.mixer.proofs.Concatenator;
import meerkat.mixer.proofs.SigmaProtocol;
import org.factcenter.qilin.primitives.RandomOracle;

import java.math.BigInteger;

/**
 * Transform a Sigma protocol into a NIZK using Fiat-Shamir
 */
public class SigmaFiatShamir<NIZKMsgType, FirstMsgType extends Message, FinalMessageType> {
    final RandomOracle randomOracle;
    final Concatenator.Pair<NIZKMsgType, FirstMsgType, FinalMessageType> concat;

    public SigmaFiatShamir(Concatenator.Pair<NIZKMsgType, FirstMsgType, FinalMessageType> concat, RandomOracle randomOracle) {
        this.concat = concat;
        this.randomOracle = randomOracle;
    }

    /**
     * Fiat–Shamir heuristic
     * @param input - protobuf contains all parameters from the first step of the current proof
     * @return randomOracle.hash(input)
     */
    public static BigInteger hash(Message input, RandomOracle randomOracle) {
        byte[] arr = input.toByteArray();
        return new BigInteger(1,randomOracle.hash(arr,arr.length));
    }

    public NIZKMsgType generateNizk(SigmaProtocol.Prover<FirstMsgType, FinalMessageType> prover) {
        FirstMsgType firstMessage = prover.getFirstMessage();
        BigInteger challenge = hash(firstMessage, randomOracle);
        FinalMessageType finalMessage = prover.getFinalMessage(challenge);
        return concat.concatenate(firstMessage, finalMessage);
    }

    public boolean verifyNizk(NIZKMsgType NIZK, SigmaProtocol.Verifier<FirstMsgType, FinalMessageType> verifier) {
        FirstMsgType firstMessage = concat.getMsg1(NIZK);
        BigInteger challenge = hash(firstMessage, randomOracle);
        return verifier.verify(firstMessage, challenge, concat.getMsg2(NIZK));
    }
}
